searchNodes=[{"doc":"First-class match specification tooling for Elixir. Matcha offers tight integration with Elixir and match specifications. Match specifications are a BEAM VM feature that executes simple pattern matching operations very close-to-the-metal, often several thousand times more performant than a comparable Enum operation . They can be used to efficiently: filter/map in-memory data find ETS objects trace specific function calls However, they are notoriously difficult to compose and use. Matcha makes this intuitive with ergonomic macros and a fluent API with which to manipulate them. Examples require Matcha # Turns Elixir code into a match specification iex&gt; spec = Matcha . spec do ...&gt; { x , y , z } -&gt; x + y + z ...&gt; end ...&gt; spec . source [ { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$3&quot; } , [ ] , [ { :+ , { :+ , :&quot;$1&quot; , :&quot;$2&quot; } , :&quot;$3&quot; } ] } ] For more information, check out the interactive usage guides , including using Matcha for: filtering &amp; mapping data selecting objects from tables tracing function calls Known Limitations Currently, it is not possible to: Use the Kernel.in/2 macro in guards. (see: open issue ) Use the Kernel.tuple_size/1 or :erlang.tuple_size/1 guards. (see: documentation ) This is a fundamental limitation of match specs. Use any is_record guards (neither Elixir's implementation because of the Kernel.tuple_size/1 limitation above, nor erlang's implementation for other reasons). (see: documentation ) Both destructure values from a data structure into bindings, and assign the datastructure to a variable, except at the top-level of a clause. This is how match specs work by design; though there may be a work-around using :erlang.map_get/2 for maps, but at this time introducing an inconsistency doesn't seem worth it.","ref":"Matcha.html","title":"Matcha","type":"module"},{"doc":"Builds a Matcha.Pattern that represents a &quot;filter&quot; operation on a given input. Match patterns represent a &quot;filter&quot; operation on a given input, ignoring anything that does not fit the match &quot;shape&quot; specified. For more information on match patterns, consult the Matcha.Pattern docs. Examples iex&gt; require Matcha ...&gt; pattern = Matcha . pattern ( { x , y , x } ) # Matcha.Pattern &lt; { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$1&quot; } &gt; iex&gt; Matcha.Pattern . match? ( pattern , { 1 , 2 , 3 } ) false iex&gt; Matcha.Pattern . match? ( pattern , { 1 , 2 , 1 } ) true","ref":"Matcha.html#pattern/1","title":"Matcha.pattern/1","type":"macro"},{"doc":"Builds a Matcha.Spec that represents a &quot;filter+map&quot; operation on a given input. The context may be :filter_map , :table , :trace , or a Matcha.Context module. This is detailed in the Matcha.Context docs. For more information on match specs, consult the Matcha.Spec docs. Examples iex&gt; require Matcha ...&gt; Matcha . spec do ...&gt; { x , y , x } ...&gt; when x &gt; y and y &gt; 0 ...&gt; -&gt; x ...&gt; { x , y , y } ...&gt; when x &lt; y and y &lt; 0 ...&gt; -&gt; y ...&gt; end # Matcha.Spec &lt; [ { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$1&quot; } , [ { :andalso , { :&gt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&gt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$1&quot; ] } , { { :&quot;$1&quot; , :&quot;$2&quot; , :&quot;$2&quot; } , [ { :andalso , { :&lt; , :&quot;$1&quot; , :&quot;$2&quot; } , { :&lt; , :&quot;$2&quot; , 0 } } ] , [ :&quot;$2&quot; ] } ] , context : :filter_map &gt;","ref":"Matcha.html#spec/2","title":"Matcha.spec/2","type":"macro"},{"doc":"Traces function calls to module , executing a spec on matching arguments. Tracing is a powerful feature of the BEAM VM, allowing for near zero-cost monitoring of what is happening in running systems. The functions in Matcha.Trace provide utilities for accessing this functionality. One of the most powerful forms of tracing uses match specifications: rather that just print information on when a certain function signature with some number of arguments is invoked, they let you: dissect the arguments in question with pattern-matching and guards take special actions in response (documented in Matcha.Context.Trace ) This macro is a shortcut for constructing a spec with the :trace context via Matcha.spec/2 , and tracing the specified module and function with it via Matcha.Trace.calls/4 . For more information on tracing in general, consult the Matcha.Trace docs. Examples iex&gt; require Matcha ...&gt; Matcha . trace_calls ( Enum , :join , limit : 3 ) do ...&gt; [ _enumerable ] -&gt; message ( &quot;using default joiner&quot; ) ...&gt; [ _enumerable , &quot;&quot; ] -&gt; message ( &quot;using default joiner (but explicitly)&quot; ) ...&gt; [ _enumerable , _custom ] -&gt; message ( &quot;using custom joiner&quot; ) ...&gt; end ...&gt; Enum . join ( 1 .. 3 ) # Prints a trace message with &quot;using default joiner&quot; appended &quot;123&quot; iex&gt; Enum . join ( 1 .. 3 , &quot;&quot; ) # Prints a trace message with &quot;using default joiner (but explicitly)&quot; appended &quot;123&quot; iex&gt; Enum . join ( 1 .. 3 , &quot;, &quot; ) # Prints a trace message with &quot;using custom joiner&quot; appended &quot;1, 2, 3&quot;","ref":"Matcha.html#trace_calls/4","title":"Matcha.trace_calls/4","type":"macro"},{"doc":"About patterns.","ref":"Matcha.Pattern.html","title":"Matcha.Pattern","type":"module"},{"doc":"","ref":"Matcha.Pattern.html#filter/2","title":"Matcha.Pattern.filter/2","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#match!/2","title":"Matcha.Pattern.match!/2","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#match?/2","title":"Matcha.Pattern.match?/2","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#to_spec/2","title":"Matcha.Pattern.to_spec/2","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#validate/1","title":"Matcha.Pattern.validate/1","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#validate!/1","title":"Matcha.Pattern.validate!/1","type":"function"},{"doc":"","ref":"Matcha.Pattern.html#t:t/0","title":"Matcha.Pattern.t/0","type":"type"},{"doc":"About specs.","ref":"Matcha.Spec.html","title":"Matcha.Spec","type":"module"},{"doc":"Uses a spec to filter out and manipulate elements of an enumerable . Elements of the enumerable that match one of the spec 's clauses will transformed as instructed. Elements that do not match will be filtered out of the result. Always returns a list. Examples iex&gt; require Matcha ...&gt; spec = Matcha . spec do ...&gt; { amount , tax } when is_integer ( amount ) and amount &gt; 0 -&gt; { :credit , amount + tax } ...&gt; end ...&gt; Matcha.Spec . filter_map ( spec , [ ...&gt; { 9001 , 0 } , ...&gt; { - 200 , - 2.50 } , ...&gt; { - 3 , - 0.5 } , ...&gt; { :error , &quot;bank was offline&quot; } , ...&gt; { 100 , 0 } , ...&gt; { - 743 , - 16.0 } , ...&gt; ] ) [ credit : 9001 , credit : 100 ] Note This function converts the enumerable to a list, which will trigger full enumeration of things like lazy Stream s. If used with an infinite stream, it will run forever! Consider using stream/2 if you need lazy filter/mapping. It isn't as efficient, but plays nicer with infinite streams, and fits into the Stream APIs.","ref":"Matcha.Spec.html#filter_map/2","title":"Matcha.Spec.filter_map/2","type":"function"},{"doc":"","ref":"Matcha.Spec.html#run/2","title":"Matcha.Spec.run/2","type":"function"},{"doc":"","ref":"Matcha.Spec.html#run!/2","title":"Matcha.Spec.run!/2","type":"function"},{"doc":"Produces a Stream that filters out and manipulates elements of an enumerable . Elements of the enumerable that match one of the spec 's clauses will transformed as instructed. Elements that do not match will be filtered out of the result. Always returns a lazy Stream enumerable. Examples iex&gt; require Matcha ...&gt; spec = Matcha . spec do ...&gt; { amount , tax } when is_integer ( amount ) and amount &lt; 0 -&gt; { :charge , amount + tax } ...&gt; end ...&gt; Matcha.Spec . stream ( spec , [ ...&gt; { 9001 , 0 } , ...&gt; { - 200 , - 2.50 } , ...&gt; { - 3 , - 0.5 } , ...&gt; { :error , &quot;bank was offline&quot; } , ...&gt; { 100 , 0 } , ...&gt; { - 743 , - 16.0 } , ...&gt; ] ) ...&gt; |&gt; Stream . take ( 2 ) ...&gt; |&gt; Enum . to_list [ charge : - 202.5 , charge : - 3.5 ] Note This function wraps the enumerable in a lazy Stream . If the enumerable is something you can safely convert to a list without going on forever or loading too much into memory, consider using filter_map/2 instead, as it is much more efficient.","ref":"Matcha.Spec.html#stream/2","title":"Matcha.Spec.stream/2","type":"function"},{"doc":"","ref":"Matcha.Spec.html#to_pattern/1","title":"Matcha.Spec.to_pattern/1","type":"function"},{"doc":"","ref":"Matcha.Spec.html#valid?/1","title":"Matcha.Spec.valid?/1","type":"function"},{"doc":"","ref":"Matcha.Spec.html#validate/1","title":"Matcha.Spec.validate/1","type":"function"},{"doc":"","ref":"Matcha.Spec.html#validate!/1","title":"Matcha.Spec.validate!/1","type":"function"},{"doc":"","ref":"Matcha.Spec.html#t:t/0","title":"Matcha.Spec.t/0","type":"type"},{"doc":"About tracing.","ref":"Matcha.Trace.html","title":"Matcha.Trace","type":"module"},{"doc":"Checks if pid is awaiting trace messages. Waits timeout milliseconds for the pid to report that all trace messages intended for it when awaiting_messages?/2 was called have been delivered. Returns true if no response is received within timeout , and you may assume that pid is still working through trace messages it has received. If it receives confirmation before the timeout , returns false . The pid must refer to an alive (or previously alive) process from the same node this function is called from , or it will raise an ArgumentError . If the atom :all is provided instead of a pid , this function returns true if any process on the current node is awaiting trace messages. This function is best used when shutting down processes (or the current node), to give them a chance to finish any tracing they are handling.","ref":"Matcha.Trace.html#awaiting_messages?/2","title":"Matcha.Trace.awaiting_messages?/2","type":"function"},{"doc":"Trace function calls to module with specified arguments . arguments may be: an integer arity, only tracing function calls with that number of parameters a Matcha.Spec , only tracing function calls whose arguments match the provided patterns If calling with just an arity, all matching calls will print a corresponding trace message. If calling with a spec, additional operations can be performed, as documented in Matcha.Context.Trace . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#calls/4","title":"Matcha.Trace.calls/4","type":"function"},{"doc":"Trace all function calls to module . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#function/3","title":"Matcha.Trace.function/3","type":"function"},{"doc":"","ref":"Matcha.Trace.html#info/2","title":"Matcha.Trace.info/2","type":"function"},{"doc":"Trace all calls to a module . By default, only 1 calls will be traced. More calls can be traced by providing an integer :limit in the opts . All other opts are forwarded to :recon_trace.calls/3 as the third argument.","ref":"Matcha.Trace.html#module/2","title":"Matcha.Trace.module/2","type":"function"},{"doc":"Stops all tracing at once.","ref":"Matcha.Trace.html#stop/0","title":"Matcha.Trace.stop/0","type":"function"},{"doc":"","ref":"Matcha.Trace.html#t:info_flag/0","title":"Matcha.Trace.info_flag/0","type":"type"},{"doc":"","ref":"Matcha.Trace.html#t:info_item/0","title":"Matcha.Trace.info_item/0","type":"type"},{"doc":"","ref":"Matcha.Trace.html#t:info_item_result/0","title":"Matcha.Trace.info_item_result/0","type":"type"},{"doc":"","ref":"Matcha.Trace.html#t:info_result/0","title":"Matcha.Trace.info_result/0","type":"type"},{"doc":"","ref":"Matcha.Trace.html#t:info_subject/0","title":"Matcha.Trace.info_subject/0","type":"type"},{"doc":"","ref":"Matcha.Trace.html#t:t/0","title":"Matcha.Trace.t/0","type":"type"},{"doc":"Different types of match spec are intended to be used for different purposes, and support different instructions in their bodies for different use-cases. The modules implementing the Matcha.Context behaviour define the different types of Matcha.Spec , provide documentation for what specialized instructions that type supports, and are used during Elixir-to-match spec conversion as a concrete function definition to use when expanding instructions (since most of these specialized instructions do not exist anywhere as an actual functions, this lets the Elixir compiler complain about invalid instructions as UndefinedFunctionError s). Predefined contexts Currently there are three applications of match specs supported: :filter_map : Matchspecs intended to be used to filter/map over a list in an optimized fashion. These types of match spec reference the Matcha.Context.FilterMap module. :table : Matchspecs intended to be used to efficiently select data from BEAM VM &quot;table&quot; tools, such as :ets , :dets , and :mnesia , and massage the values returned. These types of match spec reference the Matcha.Context.Table module. :trace : Matchspecs intended to be used to instruct tracing utilities such as :dbg and :recon_trace exactly what function calls with what arguments to trace, and allows invoking special trace command instructions in response. These types of match spec reference the Matcha.Context.Trace module. Custom contexts The context mechanism is technically extensible: any module can implement the Matcha.Context behaviour, define the callbacks, and list public no-op functions to allow their usage in specs compiled with that context (via Matcha.spec(CustomContext) do... ). In practice there is little point in defining a custom context: the supported use-cases for match specs are tightly coupled to the erlang language, and Matcha covers all of them with its provided contexts, which should be sufficient for any application. The module+behaviour+callback implementation used in Matcha is less about offering extensibility, but instead used to simplify special-casing in Matcha.Spec function implementations, raise Elixir-flavored errors when an invalid instruction is used in the different types of spec, and provide a place to document what they do when invoked.","ref":"Matcha.Context.html","title":"Matcha.Context","type":"behaviour"},{"doc":"","ref":"Matcha.Context.html#c:__context_name__/0","title":"Matcha.Context.__context_name__/0","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__default_test_target__/0","title":"Matcha.Context.__default_test_target__/0","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__emit_test_result__/1","title":"Matcha.Context.__emit_test_result__/1","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__erl_test_type__/0","title":"Matcha.Context.__erl_test_type__/0","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__handle_erl_test_results__/1","title":"Matcha.Context.__handle_erl_test_results__/1","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__invalid_test_target_error_message__/1","title":"Matcha.Context.__invalid_test_target_error_message__/1","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__prepare_source__/1","title":"Matcha.Context.__prepare_source__/1","type":"callback"},{"doc":"","ref":"Matcha.Context.html#c:__valid_test_target__/1","title":"Matcha.Context.__valid_test_target__/1","type":"callback"},{"doc":"","ref":"Matcha.Context.html#t:t/0","title":"Matcha.Context.t/0","type":"type"},{"doc":"Functions and operators that any match specs can use in their bodies. Limitations Neither tuple_size/2 nor is_record/2 are available here, though you'd expect them to be. For various reasons, Matcha cannot support is_record/2 and erlang does not support tuple_size/2 . Defined functions Note that this list corresponds to key functions in the :erlang module, or erlang operators, not their equivalents in Elixir's Kernel module (or the Bitwise guards). Allowed Elixir functions, operators, and macros composed of them are first expanded to erlang variants before they are looked up in this context. For example, Kernel.send/2 expands to erlang's ! operator, so is defined in this module as !/2. Further reading Aside from the above limitations, the common functions allowed in all match specs are just identical to those allowed in guards; so for an Elixir-ized, pre-erlang-ized expansion reference on what functions and operators you can use in any match spec, consult the docs for what is allowed in guards . For an erlang reference, see the tracing match spec docs .","ref":"Matcha.Context.Common.html","title":"Matcha.Context.Common","type":"module"},{"doc":"All match specs can call erlang's !/2 .","ref":"Matcha.Context.Common.html#!/2","title":"Matcha.Context.Common.!/2","type":"function"},{"doc":"All match specs can call erlang's */2 .","ref":"Matcha.Context.Common.html#*/2","title":"Matcha.Context.Common.*/2","type":"function"},{"doc":"All match specs can call erlang's +/1 .","ref":"Matcha.Context.Common.html#+/1","title":"Matcha.Context.Common.+/1","type":"function"},{"doc":"All match specs can call erlang's +/2 .","ref":"Matcha.Context.Common.html#+/2","title":"Matcha.Context.Common.+/2","type":"function"},{"doc":"All match specs can call erlang's -/1 .","ref":"Matcha.Context.Common.html#-/1","title":"Matcha.Context.Common.-/1","type":"function"},{"doc":"All match specs can call erlang's -/2 .","ref":"Matcha.Context.Common.html#-/2","title":"Matcha.Context.Common.-/2","type":"function"},{"doc":"All match specs can call erlang's //2 .","ref":"Matcha.Context.Common.html#//2","title":"Matcha.Context.Common.//2","type":"function"},{"doc":"All match specs can call erlang's /=/2 .","ref":"Matcha.Context.Common.html#/=/2","title":"Matcha.Context.Common./=/2","type":"function"},{"doc":"All match specs can call erlang's &lt;/2 .","ref":"Matcha.Context.Common.html#%3C/2","title":"Matcha.Context.Common.</2","type":"function"},{"doc":"All match specs can call erlang's =/=/2 .","ref":"Matcha.Context.Common.html#=/=/2","title":"Matcha.Context.Common.=/=/2","type":"function"},{"doc":"All match specs can call erlang's =:=/2 .","ref":"Matcha.Context.Common.html#=:=/2","title":"Matcha.Context.Common.=:=/2","type":"function"},{"doc":"All match specs can call erlang's =&lt;/2 .","ref":"Matcha.Context.Common.html#=%3C/2","title":"Matcha.Context.Common.=</2","type":"function"},{"doc":"All match specs can call erlang's ==/2 .","ref":"Matcha.Context.Common.html#==/2","title":"Matcha.Context.Common.==/2","type":"function"},{"doc":"All match specs can call erlang's &gt;/2 .","ref":"Matcha.Context.Common.html#%3E/2","title":"Matcha.Context.Common.>/2","type":"function"},{"doc":"All match specs can call erlang's &gt;=/2 .","ref":"Matcha.Context.Common.html#%3E=/2","title":"Matcha.Context.Common.>=/2","type":"function"},{"doc":"All match specs can call erlang's abs/1 .","ref":"Matcha.Context.Common.html#abs/1","title":"Matcha.Context.Common.abs/1","type":"function"},{"doc":"All match specs can call erlang's and/2 .","ref":"Matcha.Context.Common.html#and/2","title":"Matcha.Context.Common.and/2","type":"function"},{"doc":"All match specs can call erlang's andalso/2 .","ref":"Matcha.Context.Common.html#andalso/2","title":"Matcha.Context.Common.andalso/2","type":"function"},{"doc":"All match specs can call erlang's band/2 .","ref":"Matcha.Context.Common.html#band/2","title":"Matcha.Context.Common.band/2","type":"function"},{"doc":"All match specs can call erlang's binary_part/2 .","ref":"Matcha.Context.Common.html#binary_part/2","title":"Matcha.Context.Common.binary_part/2","type":"function"},{"doc":"All match specs can call erlang's binary_part/3 .","ref":"Matcha.Context.Common.html#binary_part/3","title":"Matcha.Context.Common.binary_part/3","type":"function"},{"doc":"All match specs can call erlang's bit_size/1 .","ref":"Matcha.Context.Common.html#bit_size/1","title":"Matcha.Context.Common.bit_size/1","type":"function"},{"doc":"All match specs can call erlang's bnot/1 .","ref":"Matcha.Context.Common.html#bnot/1","title":"Matcha.Context.Common.bnot/1","type":"function"},{"doc":"All match specs can call erlang's bor/2 .","ref":"Matcha.Context.Common.html#bor/2","title":"Matcha.Context.Common.bor/2","type":"function"},{"doc":"All match specs can call erlang's bsl/2 .","ref":"Matcha.Context.Common.html#bsl/2","title":"Matcha.Context.Common.bsl/2","type":"function"},{"doc":"All match specs can call erlang's bsr/2 .","ref":"Matcha.Context.Common.html#bsr/2","title":"Matcha.Context.Common.bsr/2","type":"function"},{"doc":"All match specs can call erlang's bxor/2 .","ref":"Matcha.Context.Common.html#bxor/2","title":"Matcha.Context.Common.bxor/2","type":"function"},{"doc":"All match specs can call erlang's byte_size/1 .","ref":"Matcha.Context.Common.html#byte_size/1","title":"Matcha.Context.Common.byte_size/1","type":"function"},{"doc":"All match specs can call erlang's ceil/1 .","ref":"Matcha.Context.Common.html#ceil/1","title":"Matcha.Context.Common.ceil/1","type":"function"},{"doc":"All match specs can call erlang's div/2 .","ref":"Matcha.Context.Common.html#div/2","title":"Matcha.Context.Common.div/2","type":"function"},{"doc":"All match specs can call erlang's element/2 .","ref":"Matcha.Context.Common.html#element/2","title":"Matcha.Context.Common.element/2","type":"function"},{"doc":"All match specs can call erlang's float/1 .","ref":"Matcha.Context.Common.html#float/1","title":"Matcha.Context.Common.float/1","type":"function"},{"doc":"All match specs can call erlang's floor/1 .","ref":"Matcha.Context.Common.html#floor/1","title":"Matcha.Context.Common.floor/1","type":"function"},{"doc":"All match specs can call erlang's hd/1 .","ref":"Matcha.Context.Common.html#hd/1","title":"Matcha.Context.Common.hd/1","type":"function"},{"doc":"All match specs can call erlang's is_atom/1 .","ref":"Matcha.Context.Common.html#is_atom/1","title":"Matcha.Context.Common.is_atom/1","type":"function"},{"doc":"All match specs can call erlang's is_binary/1 .","ref":"Matcha.Context.Common.html#is_binary/1","title":"Matcha.Context.Common.is_binary/1","type":"function"},{"doc":"All match specs can call erlang's is_bitstring/1 .","ref":"Matcha.Context.Common.html#is_bitstring/1","title":"Matcha.Context.Common.is_bitstring/1","type":"function"},{"doc":"All match specs can call erlang's is_boolean/1 .","ref":"Matcha.Context.Common.html#is_boolean/1","title":"Matcha.Context.Common.is_boolean/1","type":"function"},{"doc":"All match specs can call erlang's is_float/1 .","ref":"Matcha.Context.Common.html#is_float/1","title":"Matcha.Context.Common.is_float/1","type":"function"},{"doc":"All match specs can call erlang's is_function/1 .","ref":"Matcha.Context.Common.html#is_function/1","title":"Matcha.Context.Common.is_function/1","type":"function"},{"doc":"All match specs can call erlang's is_function/2 .","ref":"Matcha.Context.Common.html#is_function/2","title":"Matcha.Context.Common.is_function/2","type":"function"},{"doc":"All match specs can call erlang's is_integer/1 .","ref":"Matcha.Context.Common.html#is_integer/1","title":"Matcha.Context.Common.is_integer/1","type":"function"},{"doc":"All match specs can call erlang's is_list/1 .","ref":"Matcha.Context.Common.html#is_list/1","title":"Matcha.Context.Common.is_list/1","type":"function"},{"doc":"All match specs can call erlang's is_map/1 .","ref":"Matcha.Context.Common.html#is_map/1","title":"Matcha.Context.Common.is_map/1","type":"function"},{"doc":"All match specs can call erlang's is_map_key/2 .","ref":"Matcha.Context.Common.html#is_map_key/2","title":"Matcha.Context.Common.is_map_key/2","type":"function"},{"doc":"All match specs can call erlang's is_number/1 .","ref":"Matcha.Context.Common.html#is_number/1","title":"Matcha.Context.Common.is_number/1","type":"function"},{"doc":"All match specs can call erlang's is_pid/1 .","ref":"Matcha.Context.Common.html#is_pid/1","title":"Matcha.Context.Common.is_pid/1","type":"function"},{"doc":"All match specs can call erlang's is_port/1 .","ref":"Matcha.Context.Common.html#is_port/1","title":"Matcha.Context.Common.is_port/1","type":"function"},{"doc":"All match specs can call erlang's is_record/3 .","ref":"Matcha.Context.Common.html#is_record/3","title":"Matcha.Context.Common.is_record/3","type":"function"},{"doc":"All match specs can call erlang's is_reference/1 .","ref":"Matcha.Context.Common.html#is_reference/1","title":"Matcha.Context.Common.is_reference/1","type":"function"},{"doc":"All match specs can call erlang's is_tuple/1 .","ref":"Matcha.Context.Common.html#is_tuple/1","title":"Matcha.Context.Common.is_tuple/1","type":"function"},{"doc":"All match specs can call erlang's length/1 .","ref":"Matcha.Context.Common.html#length/1","title":"Matcha.Context.Common.length/1","type":"function"},{"doc":"All match specs can call erlang's map_get/2 .","ref":"Matcha.Context.Common.html#map_get/2","title":"Matcha.Context.Common.map_get/2","type":"function"},{"doc":"All match specs can call erlang's map_size/1 .","ref":"Matcha.Context.Common.html#map_size/1","title":"Matcha.Context.Common.map_size/1","type":"function"},{"doc":"All match specs can call erlang's node/0 .","ref":"Matcha.Context.Common.html#node/0","title":"Matcha.Context.Common.node/0","type":"function"},{"doc":"All match specs can call erlang's node/1 .","ref":"Matcha.Context.Common.html#node/1","title":"Matcha.Context.Common.node/1","type":"function"},{"doc":"All match specs can call erlang's not/1 .","ref":"Matcha.Context.Common.html#not/1","title":"Matcha.Context.Common.not/1","type":"function"},{"doc":"All match specs can call erlang's or/2 .","ref":"Matcha.Context.Common.html#or/2","title":"Matcha.Context.Common.or/2","type":"function"},{"doc":"All match specs can call erlang's orelse/2 .","ref":"Matcha.Context.Common.html#orelse/2","title":"Matcha.Context.Common.orelse/2","type":"function"},{"doc":"All match specs can call erlang's rem/2 .","ref":"Matcha.Context.Common.html#rem/2","title":"Matcha.Context.Common.rem/2","type":"function"},{"doc":"All match specs can call erlang's round/1 .","ref":"Matcha.Context.Common.html#round/1","title":"Matcha.Context.Common.round/1","type":"function"},{"doc":"All match specs can call erlang's self/0 .","ref":"Matcha.Context.Common.html#self/0","title":"Matcha.Context.Common.self/0","type":"function"},{"doc":"All match specs can call erlang's size/1 .","ref":"Matcha.Context.Common.html#size/1","title":"Matcha.Context.Common.size/1","type":"function"},{"doc":"All match specs can call erlang's tl/1 .","ref":"Matcha.Context.Common.html#tl/1","title":"Matcha.Context.Common.tl/1","type":"function"},{"doc":"All match specs can call erlang's trunc/1 .","ref":"Matcha.Context.Common.html#trunc/1","title":"Matcha.Context.Common.trunc/1","type":"function"},{"doc":"All match specs can call erlang's xor/2 .","ref":"Matcha.Context.Common.html#xor/2","title":"Matcha.Context.Common.xor/2","type":"function"},{"doc":"Functions and operators that :filter_map match specs can use in their bodies. Specs created in the :filter_map context are unique in that they can differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value. They return :no_match in the former case, and {:matched, value} tuples in the latter, where value can be a literal false returned from a clause. No additional functions besides those defined in Matcha.Context.Common can be used in :filter_map contexts.","ref":"Matcha.Context.FilterMap.html","title":"Matcha.Context.FilterMap","type":"module"},{"doc":"Functions and operators that :table match specs can use in their bodies. The return values of specs created in the :table context do not differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value; they return {:returned, result} tuples either way. No additional functions besides those defined in Matcha.Context.Common can be used in :table contexts.","ref":"Matcha.Context.Table.html","title":"Matcha.Context.Table","type":"module"},{"doc":"Functions and operators that :trace match specs can use in their bodies. The return values of specs created in the :trace context do not differentiate between specs that fail to find a matching clause for the given input, and specs with matching clauses that literally return the false value; they return {:traced, result, flags} tuples either way. Tracing match specs offer a wide suite of instructions to drive erlang's tracing engine in response to matching certain calls. Calls to these functions in match spec bodies will, when that clause is matched, effect the documented change during tracing. These instructions are documented and type-specced here as a convenient reference. For more information, consult the erlang tracing match spec docs . In addition to general helpful informational functions, tracing supports: Trace Flags Match specs can change how tracing behaves by changing the trace flags on any process. See :erlang.trace/3 for more information. Related functions: enable_trace/1 , enabled_trace/2 , disable_trace/1 , disable_trace/2 , trace/2 , trace/3 . Sequential Tracing Match specs can be used to transfer information between processes via sequential tracing. See the erlang sequential tracing docs for more information. Related functions: is_seq_trace/0 , set_seq_token/2 , get_seq_token/0 .","ref":"Matcha.Context.Trace.html","title":"Matcha.Context.Trace","type":"module"},{"doc":"Returns the module/function/arity of the calling function. If the calling function cannot be determined, returns :undefined . This can happen with BIFs in particular.","ref":"Matcha.Context.Trace.html#caller/0","title":"Matcha.Context.Trace.caller/0","type":"function"},{"doc":"Turns off the provided trace_flag for the current process. See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#disable_trace/1","title":"Matcha.Context.Trace.disable_trace/1","type":"function"},{"doc":"Turns off the provided trace_flag for the specified pid . See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#disable_trace/2","title":"Matcha.Context.Trace.disable_trace/2","type":"function"},{"doc":"Displays the given value on stdout for debugging purposes. Always returns true .","ref":"Matcha.Context.Trace.html#display/1","title":"Matcha.Context.Trace.display/1","type":"function"},{"doc":"Turns on the provided trace_flag for the current process. See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#enable_trace/1","title":"Matcha.Context.Trace.enable_trace/1","type":"function"},{"doc":"Turns on the provided trace_flag for the specified pid . See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp and :tracer flags are not supported in this function. Always returns true .","ref":"Matcha.Context.Trace.html#enable_trace/2","title":"Matcha.Context.Trace.enable_trace/2","type":"function"},{"doc":"Works as return_trace/0 , generating an extra exception_from message on exceptions. Causes a return_from trace message to be sent upon return from the current function. Plus, if the traced function exits because of an exception, an exception_from trace message is generated, whether or not the exception is caught . If the process trace flag silent is active, the return_from and exception_from trace messages are inhibited. Always returns true . Warning : If the traced function is tail-recursive, this match specification function destroys that property. Hence, if a match specification executing this function is used on a perpetual server process, t can only be active for a limited period of time, or the emulator will eventually use all memory in the host machine and crash. If this match specification function is inhibited using process trace flag silent, tail-recursiveness still remains.","ref":"Matcha.Context.Trace.html#exception_trace/0","title":"Matcha.Context.Trace.exception_trace/0","type":"function"},{"doc":"Retreives the (opaque) value of the trace token for the current process. If the current process is not being traced, returns [] . Acts identically to :seq_trace.get_token/0 . The docs say that the return value can be passed back into :seq_trace.set_token/1 . However, in a tracing match spec context, there is no equivalent ( set_seq_token/2 works, but there's no set_seq_token/1 ). So I am unsure what this can be used for. For more information, consult :seq_trace.get_token/0 docs.","ref":"Matcha.Context.Trace.html#get_seq_token/0","title":"Matcha.Context.Trace.get_seq_token/0","type":"function"},{"doc":"Returns the value of the current node's trace control word. Identical to calling :erlang.system_info/1 with the argument :trace_control_word . The trace control word is a 32-bit unsigned integer intended for generic trace control. The trace control word can be tested and set both from within trace match specifications and with BIFs.","ref":"Matcha.Context.Trace.html#get_tcw/0","title":"Matcha.Context.Trace.get_tcw/0","type":"function"},{"doc":"Returns true if a sequential trace token is set for the current process, otherwise false .","ref":"Matcha.Context.Trace.html#is_seq_trace/0","title":"Matcha.Context.Trace.is_seq_trace/0","type":"function"},{"doc":"Sets an additional message appended to the trace message sent. One can only set one additional message in the body. Later calls replace the appended message. Always returns true . As a special case, {message, false} disables sending of trace messages ('call' and 'return_to') for this function call, just like if the match specification had not matched. This can be useful if only the side effects of the match spec clause's body part are desired. Another special case is {message, true} , which sets the default behavior, as if the function had no match specification; trace message is sent with no extra information (if no other calls to message are placed before {message, true} , it is in fact a &quot;noop&quot;).","ref":"Matcha.Context.Trace.html#message/1","title":"Matcha.Context.Trace.message/1","type":"function"},{"doc":"Returns some textual information about the current process as a binary.","ref":"Matcha.Context.Trace.html#process_dump/0","title":"Matcha.Context.Trace.process_dump/0","type":"function"},{"doc":"Causes a return_from trace message to be sent upon return from the current function. If the process trace flag silent is active, the return_from trace message is inhibited. Always returns true . Warning : If the traced function is tail-recursive, this match specification function destroys that property. Hence, if a match specification executing this function is used on a perpetual server process, it can only be active for a limited period of time, or the emulator will eventually use all memory in the host machine and crash. If this match specification function is inhibited using process trace flag silent, tail-recursiveness still remains.","ref":"Matcha.Context.Trace.html#return_trace/0","title":"Matcha.Context.Trace.return_trace/0","type":"function"},{"doc":"Sets a label, serial number, or flag token to value for sequential tracing. Acts like :seq_trace.set_token/2 , except returns true on success, and 'EXIT' on error or bad argument. Note that this function cannot be used to exclude message passing from the trace, since that is normally accomplished by passing [] into :seq_trace.set_token/1 (however there is no set_seq_token/1 allowed in match specs). Note that the values set here cannot be introspected in a match spec tracing context ( get_seq_token/0 returns an opaque representation of the current trace token, but there's no get_seq_token/1 to inspect individual values). For more information, consult :seq_trace.set_token/2 docs.","ref":"Matcha.Context.Trace.html#set_seq_token/2","title":"Matcha.Context.Trace.set_seq_token/2","type":"function"},{"doc":"Sets the value of the current node's trace control word to trace_control_word . Identical to calling :erlang.system_flag/2 with the arguments :trace_control_word and trace_control_word . Returns the previous value of the node's trace control word.","ref":"Matcha.Context.Trace.html#set_tcw/1","title":"Matcha.Context.Trace.set_tcw/1","type":"function"},{"doc":"Changes the verbosity of the current process's messaging mode . If mode is true , supresses all trace messages. If mode is false , re-enables trace messages in future calls. If mode is anything else, the current mode remains active.","ref":"Matcha.Context.Trace.html#silent/1","title":"Matcha.Context.Trace.silent/1","type":"function"},{"doc":"Atomically disables and enables a set of trace flags for the current process in one go. Flags enabled in the enable_flags list will override duplicate flags in the disable_flags list. See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp flag is not supported in this function, however unlike the enable_trace/1 and disable_trace/1 functions, the :tracer flags are supported.. If no :tracer is specified, the same tracer as the process executing the match specification is used (not the meta tracer). If that process doesn't have tracer either, then trace flags are ignored. When using a tracer module, the module must be loaded before the match specification is executed. If it is not loaded, the match fails. Returns true if any trace property was changed for the current process, otherwise false .","ref":"Matcha.Context.Trace.html#trace/2","title":"Matcha.Context.Trace.trace/2","type":"function"},{"doc":"Atomically disables and enables a set of trace flags for the given pid in one go. Flags enabled in the enable_flags list will override duplicate flags in the disable_flags list. See the third parameter :erlang.trace/3 for a list of flags and their effects. Note that the :cpu_timestamp flag is not supported in this function, however unlike the enable_trace/1 and disable_trace/1 functions, the :tracer flags are supported.. If no :tracer is specified, the same tracer as the process executing the match specification is used (not the meta tracer). If that process doesn't have tracer either, then trace flags are ignored. When using a tracer module, the module must be loaded before the match specification is executed. If it is not loaded, the match fails. Returns true if any trace property was changed for the given pid , otherwise false .","ref":"Matcha.Context.Trace.html#trace/3","title":"Matcha.Context.Trace.trace/3","type":"function"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token/0","title":"Matcha.Context.Trace.seq_token/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_component/0","title":"Matcha.Context.Trace.seq_token_component/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_current_serial_number/0","title":"Matcha.Context.Trace.seq_token_current_serial_number/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_flag/0","title":"Matcha.Context.Trace.seq_token_flag/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_label_value/0","title":"Matcha.Context.Trace.seq_token_label_value/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_previous_serial_number/0","title":"Matcha.Context.Trace.seq_token_previous_serial_number/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_serial_number/0","title":"Matcha.Context.Trace.seq_token_serial_number/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_serial_value/0","title":"Matcha.Context.Trace.seq_token_serial_value/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:seq_token_value/0","title":"Matcha.Context.Trace.seq_token_value/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:trace_flag/0","title":"Matcha.Context.Trace.trace_flag/0","type":"type"},{"doc":"","ref":"Matcha.Context.Trace.html#t:tracer_trace_flag/0","title":"Matcha.Context.Trace.tracer_trace_flag/0","type":"type"},{"doc":"Standard behaviour for Matcha errors.","ref":"Matcha.Error.html","title":"Matcha.Error","type":"behaviour"},{"doc":"","ref":"Matcha.Error.html#c:format_prelude/1","title":"Matcha.Error.format_prelude/1","type":"callback"},{"doc":"","ref":"Matcha.Error.html#format_problem/1","title":"Matcha.Error.format_problem/1","type":"function"},{"doc":"","ref":"Matcha.Error.html#c:format_source/1","title":"Matcha.Error.format_source/1","type":"callback"},{"doc":"","ref":"Matcha.Error.html#t:problem/0","title":"Matcha.Error.problem/0","type":"type"},{"doc":"","ref":"Matcha.Error.html#t:problems/0","title":"Matcha.Error.problems/0","type":"type"},{"doc":"Error raised when a Matcha.Pattern is invalid.","ref":"Matcha.Pattern.Error.html","title":"Matcha.Pattern.Error","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Matcha.Pattern.Error.html#message/1","title":"Matcha.Pattern.Error.message/1","type":"function"},{"doc":"","ref":"Matcha.Pattern.Error.html#t:t/0","title":"Matcha.Pattern.Error.t/0","type":"type"},{"doc":"Error raised when rewriting AST into a match pattern/spec.","ref":"Matcha.Rewrite.Error.html","title":"Matcha.Rewrite.Error","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Matcha.Rewrite.Error.html#message/1","title":"Matcha.Rewrite.Error.message/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.Error.html#t:t/0","title":"Matcha.Rewrite.Error.t/0","type":"type"},{"doc":"Error raised when a Matcha.Spec is invalid.","ref":"Matcha.Spec.Error.html","title":"Matcha.Spec.Error","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Matcha.Spec.Error.html#message/1","title":"Matcha.Spec.Error.message/1","type":"function"},{"doc":"","ref":"Matcha.Spec.Error.html#t:t/0","title":"Matcha.Spec.Error.t/0","type":"type"},{"doc":"Error raised when trying to trace calls.","ref":"Matcha.Trace.Error.html","title":"Matcha.Trace.Error","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"Matcha.Trace.Error.html#message/1","title":"Matcha.Trace.Error.message/1","type":"function"},{"doc":"","ref":"Matcha.Trace.Error.html#t:t/0","title":"Matcha.Trace.Error.t/0","type":"type"},{"doc":"About rewrites.","ref":"Matcha.Rewrite.html","title":"Matcha.Rewrite","type":"module"},{"doc":"","ref":"Matcha.Rewrite.html#ast_to_pattern_source/2","title":"Matcha.Rewrite.ast_to_pattern_source/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#ast_to_spec_source/2","title":"Matcha.Rewrite.ast_to_spec_source/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#binding/2","title":"Matcha.Rewrite.binding/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#bound?/2","title":"Matcha.Rewrite.bound?/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#is_call/1","title":"Matcha.Rewrite.is_call/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_invocation/1","title":"Matcha.Rewrite.is_invocation/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_literal/1","title":"Matcha.Rewrite.is_literal/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_named_var/1","title":"Matcha.Rewrite.is_named_var/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_non_literal/1","title":"Matcha.Rewrite.is_non_literal/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_remote_call/1","title":"Matcha.Rewrite.is_remote_call/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#is_var/1","title":"Matcha.Rewrite.is_var/1","type":"macro"},{"doc":"","ref":"Matcha.Rewrite.html#outer_var?/2","title":"Matcha.Rewrite.outer_var?/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#pattern_to_spec/2","title":"Matcha.Rewrite.pattern_to_spec/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#perform_expansion/2","title":"Matcha.Rewrite.perform_expansion/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#problem/1","title":"Matcha.Rewrite.problem/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#problems/1","title":"Matcha.Rewrite.problems/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#resolve_context/1","title":"Matcha.Rewrite.resolve_context/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#rewrite_bindings/2","title":"Matcha.Rewrite.rewrite_bindings/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#rewrite_body/2","title":"Matcha.Rewrite.rewrite_body/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#rewrite_conditions/2","title":"Matcha.Rewrite.rewrite_conditions/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#rewrite_match/2","title":"Matcha.Rewrite.rewrite_match/2","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#spec_to_pattern/1","title":"Matcha.Rewrite.spec_to_pattern/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#spec_to_pattern!/1","title":"Matcha.Rewrite.spec_to_pattern!/1","type":"function"},{"doc":"","ref":"Matcha.Rewrite.html#t:t/0","title":"Matcha.Rewrite.t/0","type":"type"},{"doc":"","ref":"Matcha.Rewrite.html#t:var_ast/0","title":"Matcha.Rewrite.var_ast/0","type":"type"},{"doc":"","ref":"Matcha.Rewrite.html#t:var_binding/0","title":"Matcha.Rewrite.var_binding/0","type":"type"},{"doc":"","ref":"Matcha.Rewrite.html#t:var_ref/0","title":"Matcha.Rewrite.var_ref/0","type":"type"},{"doc":"About sources.","ref":"Matcha.Source.html","title":"Matcha.Source","type":"module"},{"doc":"","ref":"Matcha.Source.html#all_matches/0","title":"Matcha.Source.all_matches/0","type":"function"},{"doc":"","ref":"Matcha.Source.html#compile/2","title":"Matcha.Source.compile/2","type":"function"},{"doc":"","ref":"Matcha.Source.html#match_all/0","title":"Matcha.Source.match_all/0","type":"function"},{"doc":"","ref":"Matcha.Source.html#run/2","title":"Matcha.Source.run/2","type":"function"},{"doc":"","ref":"Matcha.Source.html#test/2","title":"Matcha.Source.test/2","type":"function"},{"doc":"","ref":"Matcha.Source.html#t:all_matches/0","title":"Matcha.Source.all_matches/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:body/0","title":"Matcha.Source.body/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:clause/0","title":"Matcha.Source.clause/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:compiled/0","title":"Matcha.Source.compiled/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:condition/0","title":"Matcha.Source.condition/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:conditions/0","title":"Matcha.Source.conditions/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:context/0","title":"Matcha.Source.context/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:erl_test_type/0","title":"Matcha.Source.erl_test_type/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:expression/0","title":"Matcha.Source.expression/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:match_all/0","title":"Matcha.Source.match_all/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:pattern/0","title":"Matcha.Source.pattern/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:spec/0","title":"Matcha.Source.spec/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:test_result/0","title":"Matcha.Source.test_result/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:test_target/0","title":"Matcha.Source.test_target/0","type":"type"},{"doc":"","ref":"Matcha.Source.html#t:trace_flags/0","title":"Matcha.Source.trace_flags/0","type":"type"},{"doc":"","ref":"usage.html","title":"Using Matcha","type":"extras"},{"doc":"The latest version of Matcha can be installed in scripts or iex via: Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] ) IO . puts ( &quot;Installed matcha version: \#{ Application . spec ( :matcha , :vsn ) } &quot; ) The primary way to build Matcha constructs are through the macros in the Matcha module, so we require it to make sure we can use them: require Matcha spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"usage.html#getting-started","title":"Using Matcha - Getting started","type":"extras"},{"doc":"","ref":"filtering-and-mapping.html","title":"...for Filtering/Mapping","type":"extras"},{"doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] ) require Matcha spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"filtering-and-mapping.html#getting-started","title":"...for Filtering/Mapping - Getting started","type":"extras"},{"doc":"","ref":"tables.html","title":"...for ETS/DETS/Mnesia","type":"extras"},{"doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] ) require Matcha spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"tables.html#getting-started","title":"...for ETS/DETS/Mnesia - Getting started","type":"extras"},{"doc":"","ref":"tracing.html","title":"...for Tracing","type":"extras"},{"doc":"Mix . install ( [ { :matcha , github : &quot;christhekeele/matcha&quot; , tag : &quot;stable&quot; } ] ) require Matcha spec = Matcha . spec do { x , y , z } -&gt; { x , y , z } end","ref":"tracing.html#getting-started","title":"...for Tracing - Getting started","type":"extras"},{"doc":"Matcha uses Semantic Versioning 2.0.0 .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Thanks for considering contributing to Matcha!","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"We cultivate a set of issues that are good ways to contribute to Matcha for the first time. Looking for other ways to contribute? Consider: Improving the documentation Documentation is the most valuable contribution you can make directly to the repository! It is easy to overlook documentation when programming, and difficult to look back on something you understand and see where others may get confused. This makes it hard to keep documentation high-quality, and all assistance in fighting entropy is invaluable! Tackling something that doesn't require extensive knowledge of the codebase These are less-involved issues that should be approachable without spending a bunch of time studying the entire project. They generally touch parts of the library that are similar across other Elixir and open-source projects. Addressing regressions in upcoming releases to the language Matcha continuously looks ahead to upcoming language releases, running its test suite against them to anticipate compatibility issues. If a storm is brewing on the horizon, the bleeding-edge test suite normally catches it, with more than enough time for someone to jump in and deal with the regression. Submitting a PR See a typo or a broken link? Is existing documentation unclear, or does following it lead to behaviour you consider surprising? Jump in and help us correct it! Starting a discussion Have an idea for an improvement or an enhancement, but don't see a issue for it yet? Crack open a discussion and flesh it out with the maintainers! Opening a new issue Is something else the matter, or do the ideas above not fit what you have in mind? Create an issue and continue the conversation!","ref":"contributing.html#where-to-contribute","title":"Contributing - Where to Contribute","type":"extras"},{"doc":"Want to contribute code? Here's what you need to know. Reserved Branches &amp; Tags latest is the integration branch where work comes together. This means that you can get the &quot;cutting edge&quot; version of Matcha via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;latest&quot; ] release is the staging branch where code intended for the next release is placed. This means that you can get the &quot;release candidate&quot; version of Matcha via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;release&quot; ] stable is the floating pointer to the &quot;highest&quot; semantic version of Matcha released to hex.pm . This means that the &quot;latest official&quot; version of Matcha is available identically via: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;stable&quot; ] and Mix . install matcha : &quot;&gt;= 0.0.0&quot; tags starting with v , ex vX.Y.Z-mayberc , represent versions published to hex.pm . If versions must be modified or yanked, currently these tags must be deleted or moved manually. This means that these two are equivalent: Mix . install matcha : [ github : &quot;christhekeele/matcha&quot; , ref : &quot;vX.Y.Z-mayberc&quot; ] and Mix . install matcha : &quot;vX.Y.Z-mayberc&quot; All other branch or tag names are fair game. Project Structure Matcha is a pretty standard Elixir library, and should be navigable to anyone familiar with such things. Here is the map, with points of interest where it may deviate from a typical project: matcha │ ├ ─ ─ CONTRIBUTING . md # YOU ARE HERE │ ├ ─ ─ README . md # Project landingpad ├ ─ ─ mix . exs # Project manifest │ ├ ─ ─ VERSION # Library version ├ ─ ─ lib / # Library source code │ ├ ─ ─ test / # Test suite │ ├ ─ ─ unit / # Tests modules as laid out in lib/ │ └ ─ ─ usecases / # Tests derived from realworld ms usage │ ├ ─ ─ docs / # Extra material for docgen │ ├ ─ ─ img / # Images used in docgen │ └ ─ ─ guides / # Interactive livebook guides │ ├ ─ ─ CHANGELOG . md # Describes changes in each release │ └ ─ ─ LICENSE . md # License Matcha is available under","ref":"contributing.html#code","title":"Contributing - Code","type":"extras"},{"doc":"Guides are maintained in the docs/guides folder and built with livebook . I recommend developing them against an actual local instance of livebook, via LIVEBOOK_TOKEN_ENABLED=false livebook server --root-path docs/guides . Avoid having the .livemd files also open in an editor as you work on a guide, to avoid getting into save-tug-of-war.","ref":"contributing.html#guides","title":"Contributing - Guides","type":"extras"},{"doc":"Checks Matcha has three different checks that may run during various automatic builds. If you want to get ahead of build failures, you can run them all locally before pushing up code with the command mix checks . (This is the default task, so you can simply invoke it via mix .) This is equivalent to running: mix test Runs the tests found in test/ , checking for failures. mix lint Checks for compiler warnings, formatting divergences, and style problems . mix typecheck Runs dialyzer , checking for provable type issues. Suites Matcha has 5 test suites that run different checks automatically, depending on what's happening, for different versions of erlang/OTP, Elixir, and Matcha's dependencies. Versions The sets of versions we run checks against are named: preferred otp : The latest minor version of the highest major version erlang we want to support elixir : The latest patch version of highest minor Elixir we want to support deps : The locked-down version of our dependencies in our mix.lock matrix otp : The latest minor versions of every major erlang version we want to support elixir : The latest patch versions of every minor Elixir version we want to support deps : The locked-down version of our dependencies in our mix.lock edge otp : The latest major version erlang elixir : The upcoming version of Elixir available on its default branch deps : The un-locked version of dependencies in our mix.exs Workflows The automated test workflows we run are: Test Suite Runs all mix checks for the preferred versions of our dependencies. Runs on every set of commits pushed up to GitHub, on source or forked repositories. Provides continuous feedback on every potential change to the codebase. Test Status Runs the Test Suite , and updates related code quality services about its robustness. Runs on every set of commits added to the latest and release branches. Provides insight into test meta-data like code coverage, documentation quality, etc. displayed in the README.md. Test Matrix Runs all mix checks for every set of versions in our matrix of dependencies. Runs on every set of commits added to the latest branch. Provides full feedback on if each approved change will work on all supported versions. Test Release Runs the Test Matrix , and performs a dry-run of a planned release. Runs on every set of commits added to pull requests to the release branch. Provides a preview of what a release would look like if published from the release branch on all supported versions. Test Edge Runs all mix checks for the edge versions of our dependencies. Runs on UTC midnight. Provides continuous feedback on how prepared the codebase is for upstream changes in dependencies.","ref":"contributing.html#tests","title":"Contributing - Tests","type":"extras"},{"doc":"Copyright © 2020 Christopher Keele Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"}]